var http = require("http");
var buster = require("buster-core");
var responseBody = require("./util").responseBody;
var bPromise = require("buster-promise");
buster.extend(buster, require("buster-multicast"));

module.exports = {
    create: function (opt) {
        var session = Object.create(this);
        session.rootPath = opt.rootPath;
        session.host = opt.host || "localhost";
        session.port = opt.port || 1111;
        var jsonHttpClient = buster.jsonHttpClient.create(session.host, session.port);

        if (opt.multicastUrl) {
            session.multicaster = buster.multicastClient.create({
                httpClient: jsonHttpClient
            });

            session.multicaster.url = opt.multicastUrl;
            session.multicaster.id = opt.multicastClientId;
        }

        return session;
    },

    connect: function () {
        if (this.multicaster) {
            this.multicaster.listen();
        }
    },

    createFromHttpResponse: function (res, opt) {
        var promise = bPromise.create();

        responseBody(res, function (body) {
            try {
                var session = this.create(buster.extend(JSON.parse(body), opt));
                session.connect();
            } catch (e) {
                promise.reject(e);
            }
                promise.resolve(session);
        }.bind(this));

        return promise;
    },

    on: function (event, callback) {
        if (!this.multicaster) {
            throw new TypeError("Session has no multicaster");
        }

        return this.multicaster.on(event, callback);
    },

    emit: function (event, data) {
        if (!this.multicaster) {
            throw new TypeError("Session has no multicaster");
        }

        return this.multicaster.emit(event, data);
    },

    onMessage: function (callback) {
        if (!this.multicaster) {
            throw new TypeError("Session has no multicaster");
        }

        return this.multicaster.onMessage(callback);
    },

    close: function () {
        var disconnectMulticaster;

        if (this.multicaster) {
            disconnectMulticaster = this.multicaster.disconnect();
        }

        var promise = bPromise.create();

        bPromise.thenable(disconnectMulticaster).then(function () {
            var reqData = {
                method: "DELETE",
                host: this.host,
                port: this.port,
                path: this.rootPath
            };

            var req = http.request(reqData, function (res) { promise.resolve(); });
            req.end();

            req.on("error", function (err) {
                try { promise.reject(err); } catch (e) { /* One error is enough */ }
            });
        }.bind(this));

        return promise;
    },

    getClients: function () {
        var promise = bPromise.create();

        http.request({
            host: this.host,
            port: this.port,
            path: "/sessions/messaging/clients",
            agent: false
        }, function (res) {
            var body = "";
            res.on("data", function (chunk) { body += chunk; });

            res.on("end", function () {
                promise.resolve(JSON.parse(body));
            });
        }).end();

        return promise;
    }
};
